/*
 * Example program showing how to use the GPTimer in edge time mode
 * Uses a PWM signal to generate interrupts for the timer capture
 * Pins (PWMPIN and TIMERPIN) must be connected externally.
*/

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

#include <ti/drivers/PIN.h>
#include <ti/drivers/pin/PINCC26XX.h>

#include <ti/drivers/PWM2.h>
#include <ti/drivers/timer/GPTimerCC26XX.h>

#include "Board.h"

/* Task data */
Task_Struct timerTask;
char timerTaskStack[512];


/* Driver handles and configuration structs */
PWM_Handle hPWM;
GPTimerCC26XX_Handle hTimer;
PIN_State gpTimerPinState;
PIN_Handle gpTimerPinHandle;

#define PWMPIN IOID_25 /* LED 1 on CC2640 7x7 package */
PIN_Config pwmPin[] =  { PWMPIN | PIN_GPIO_OUTPUT_EN | PIN_GPIO_LOW | PIN_PUSHPULL | PIN_DRVSTR_MAX ,
                         PIN_TERMINATE };


#define TIMERPIN IOID_27 /* LED 2 on CC2640 7x7 package */
PIN_Config gptPin[] =  { TIMERPIN | PIN_GPIO_OUTPUT_DIS | PIN_INPUT_EN | PIN_HYSTERESIS,
                         PIN_TERMINATE };


/* Timer buffer */
#define TIMERBUFSIZE 128
uint32_t timerBufIndex = 0;
uint32_t timerBuf[TIMERBUFSIZE] = {0};

/* Timer callback, called when capture occurs */
void timerCallback(GPTimerCC26XX_Handle handle, GPTimerCC26XX_IntMask interruptMask) {
  timerBuf[timerBufIndex++] = GPTimerCC26XX_getValue(handle);
  if(timerBufIndex == TIMERBUFSIZE) {
    GPTimerCC26XX_stop(handle);
  }
}

void taskFxn(UArg a0, UArg a1)
{

  /* PWM in Hz with fractional duty 
   * Used to generate pulses for GPTimer edge time counting.
  */
  
  PWM_Params pwmParams;
  PWM_Params_init(&pwmParams);
  
  pwmParams.periodUnit  = PWM_PERIOD_HZ;
  pwmParams.periodValue = 100e3;
  pwmParams.dutyUnit    = PWM_DUTY_FRACTION; // Fractional duty cycle
  pwmParams.dutyValue   = PWM_DUTY_FRACTION_MAX/2; //50% duty
  pwmParams.idleLevel   = PWM_IDLE_LOW;

  hPWM = PWM_open(CC2650_PWM0, &pwmParams);
  PWM_start(hPWM);

  /* Set up GPTimer 0B in edge time capture mode */
  GPTimerCC26XX_Params timerParams;
  GPTimerCC26XX_Params_init(&timerParams);
  timerParams.mode  = GPT_MODE_EDGE_TIME_UP;
  timerParams.width = GPT_CONFIG_16BIT;;
  
  hTimer = GPTimerCC26XX_open(CC2650_GPTIMER0B, &timerParams);

  /* Register interrupt when capture happens */
  GPTimerCC26XX_registerInterrupt(hTimer, timerCallback, GPT_INT_CAPTURE);

  /* Open pin handle and route pin to timer */
  gpTimerPinHandle = PIN_open(&gpTimerPinState, gptPin);
  GPTimerCC26XX_PinMux pinMux = GPTimerCC26XX_getPinMux(hTimer);
  PINCC26XX_setMux(gpTimerPinHandle, PIN_ID(TIMERPIN), pinMux);

  /* Start timer */
  GPTimerCC26XX_start(hTimer);

  while (1) 
  {
    Task_sleep(BIOS_WAIT_FOREVER);
  }
}

int main(void)
{
  /* Pin init - no pins initialized */
  PIN_Config pinConfig[] = {PIN_TERMINATE};
  PIN_init(pinConfig);

  Task_Params taskParams;


  /* Configure the OS task */
  Task_Params_init(&taskParams);
  taskParams.stack = timerTaskStack;
  taskParams.stackSize = sizeof(timerTaskStack);
  taskParams.priority = 1;
  Task_construct(&timerTask, taskFxn, &taskParams, NULL);

  /* Start TI-RTOS */
  BIOS_start();

  return 0;
}
